{{!
   This file is part of Moodle - http://moodle.org/

   Moodle is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Moodle is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
   }}
{{!
   @template mod_assign/timelimit_panel

   Show a timer that counts down to 0.

   Classes required for JS:
   * None

   Data attibutes required for JS:
   * None

   Example context (json):
   {
       "timerstartvalue": 1729000
   }

}}
<div{{!
    }} id="mod_assign-timer-{{uniqid}}"{{!
    }} role="timer" aria-atomic="true"{{!
    }} aria-relevant="text"{{!
    }} data-starttime="{{timerstartvalue}}"{{!
}}>
</div>
{{#js}}
require(['mod_assign/timer'], function(Timer) {
    Timer.init("mod_assign-timer-{{uniqid}}");
});
{{/js}}
